/*
 * This library provides the functionnality of reading NFC tags 
 * using J2ME.
 *
 * Copyright (C) 2009-2011, DECARTE (INRIA)
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library in the file LICENSE.TXT; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */


import java.util.*;
import org.xml.sax.*;
import org.xml.sax.helpers.*;

class XmlHandler extends DefaultHandler {
    // this will hold all the data we read
    private Vector includes;
    private Vector excludes;
    boolean IsInclude = false;
    boolean IsExclude = false;
    
    public XmlHandler() {
    	includes = new Vector();
    	excludes = new Vector();
    }
 
    public Vector getInclude() {
        return includes;
    }
 
    public Vector getExclude() {
        return excludes;
    }
    
 
    // XML EVENT PROCESSING METHODS (DEFINED BY DefaultHandler)
 
    // startElement is the opening part of the tag "<tagname...>"
    public void startElement(String uri, String localName, String qName, Attributes attributes) throws SAXException {
        if (qName.equals("includePattern")) {
        	IsInclude = true;} 
        if (qName.equals("excludePattern")) {
        	IsExclude = true;} 
     
    }
 
    // endElement is the closing part ("</tagname>"), or the opening part if it ends with "/>"
    // so, a tag in the form "<tagname/>" generates both startElement() and endElement()
    public void endElement(String uri, String localName, String qName) throws SAXException {
    	IsInclude = false;
    	IsExclude = false;
    }
 
    // "characters" are the text inbetween tags
    public void characters(char[] ch, int start, int length) throws SAXException {
        // we're only interested in this inside a <includePattern.../> tag
        if (IsInclude) {
        	String tempo = new String(ch, start, length);
        	includes.addElement(tempo);
        	 System.out.println("includes.add:"+tempo);
        	
        }
        if (IsExclude) {
        	String tempo = new String(ch, start, length);
        	excludes.addElement(tempo);
        	System.out.println("excludes.add:"+tempo);
        }
    }
}
