/*
 * This library provides the functionnality of writing NFC tags 
 * using J2ME.
 *
 * Copyright (C) 2009-2011, DECARTE (INRIA)
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library in the file LICENSE.TXT; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

// Packages for contactless Communcation
import java.io.IOException;
import javax.microedition.contactless.ContactlessException;
import javax.microedition.contactless.DiscoveryManager;
import javax.microedition.contactless.TargetProperties;
import javax.microedition.contactless.TargetListener;
import javax.microedition.contactless.TargetType;

// Packages for NDEF
import javax.microedition.contactless.ndef.NDEFRecord;
import javax.microedition.contactless.ndef.NDEFMessage;
import javax.microedition.contactless.ndef.NDEFRecordType;
import javax.microedition.contactless.ndef.NDEFTagConnection;
import javax.microedition.io.Connector;


// Packages for GUI Stuff
import javax.microedition.lcdui.Alert;
import javax.microedition.lcdui.AlertType;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Form;
import javax.microedition.lcdui.StringItem;
import javax.microedition.lcdui.TextField;

import javax.microedition.midlet.*;

import javax.microedition.lcdui.Choice;
import javax.microedition.lcdui.ChoiceGroup;

public class Writer extends MIDlet implements TargetListener, CommandListener {

    // Display Stuff
    private Command exitCommand;
    private Form form;
   // private TextField SetTextField;
    private TextField TypeIfOther;
    private TextField DataField;
    private ChoiceGroup choiseGroup;
    private String TypeSelected;
    
    
    public Writer() {
        exitCommand = new Command("Exit", Command.EXIT, 1);
        form = new Form("NFC demo: Writer");

       form.addCommand(exitCommand);
        form.setCommandListener(this);
        form.append("Touch Tag to write (urn:nfc:wkt:ISO14443:type:data)");
        
        
        choiseGroup = new ChoiceGroup ("Type:", Choice.EXCLUSIVE,new String[]
                                            {"ID", "price", "URL", "other"}, null);
        choiseGroup.setSelectedIndex(0, true);
        form.append(choiseGroup);

        TypeIfOther = new TextField("Input 'type' if 'other' is chosen:", "", 255, TextField.ANY);
        form.append(TypeIfOther);
        DataField = new TextField("'data' field:", "", 255, TextField.ANY);
        form.append(DataField);
        
        
        // Registration of the TargetListener for external contactless
        // Targets (in this RFID_TAG).
        try {
            DiscoveryManager dm = DiscoveryManager.getInstance();
            dm.addTargetListener(this, TargetType.NDEF_TAG);

        } catch (ContactlessException ce) {
            displayAlert("Unable to register TargetListener: " + ce.toString(), AlertType.ERROR);
        }

    }

    public void startApp() {
        Display.getDisplay(this).setCurrent(form);
    }

    public void pauseApp() {
    }

    public void destroyApp(boolean unconditional) {

    }

    /**
     * Implementation of the Call-Back Function of the TargetListener
     * @param targetProperties: Array of Targets found by the Phone
     *
     */
    public void targetDetected(TargetProperties[] targetProperties) {

        // in case no targets found, exit the method
        if (targetProperties.length == 0) {
            return;
        }

        // NDEF Connection for write Operation
        NDEFTagConnection ndconn = getNDEFTAGConnection(targetProperties);
        
        try {
            // Open NDEF Connection to the first Tag found
            // ndconn = (NDEFTagConnection) Connector.open(targetProperties[0].getUrl());


            // Headerbyte: open Bookmark in Browser
            byte[] headerByte = {0x00};

            for(int i=0;i<4;i++) {
                if(choiseGroup.isSelected(i)) {
                	TypeSelected = choiseGroup.getString(i);
                }
            }
            
            
            if ((TypeSelected.equalsIgnoreCase("other"))&& (TypeIfOther.getString().equalsIgnoreCase("")))
            {displayAlert("Please input 'type' if 'other' is chosen.", AlertType.INFO);
            	 return; }
            
            if (DataField.getString().equalsIgnoreCase(""))
            {displayAlert("Please input 'data' field.", AlertType.INFO);
            	 return; }
            
            if (TypeSelected.equalsIgnoreCase("other"))
            {
            	TypeSelected = TypeIfOther.getString();
            }
            // Payload Text: URL as a byte-Array
            String StringToWrite = "urn:nfc:wkt:ISO14443:"+TypeSelected+":"+DataField.getString();
            byte[] urlBytes = StringToWrite.getBytes();
            //byte[] urlBytes ={0x00};
            	
            // Create NDEF Record
            // "urn:nfc:wkt:U"
            NDEFRecord r = new NDEFRecord(new NDEFRecordType(NDEFRecordType.NFC_FORUM_RTD, "urn:nfc:wkt:U"), null, null);
         //   NDEFRecord r = new NDEFRecord(new NDEFRecordType(NDEFRecordType.UNKNOWN , null), null, null);
            // Append Payload Manually
            //r.appendPayload(headerByte);
            r.appendPayload(urlBytes);

            // Create Recoard Array and Add Record
            NDEFRecord[] myRecArray = new NDEFRecord[]{r};

            // Generate a Message out of the Array
            NDEFMessage myMessage = new NDEFMessage(myRecArray);

            // Write Message to Tag
            ndconn.writeNDEF(myMessage);

//            byte[] tempo = ndconn.readNDEF().getRecord(0).getPayload();
//            String value = new String(tempo);
            displayAlert(" '"+StringToWrite+"' has been written to tag.", AlertType.INFO);
            TypeIfOther.setString("");
            DataField.setString("");
            choiseGroup.setSelectedIndex(0, true);

        } catch (IOException io) {
            displayAlert("IOException during open() or Write(): " + io.toString(), AlertType.ERROR);
        } catch (ContactlessException io) {
            displayAlert("ContactlessException during writeNDEF(): " + io.toString(), AlertType.ERROR);
        } catch (Exception e) {
            displayAlert("Exception: " + e.toString(), AlertType.ERROR);
        } // in case of an exception, close connection properly
        finally {

            // close Connection again
            if (ndconn != null) {
                try {
                    ndconn.close();
                } catch (IOException ex) {
                    displayAlert("IOException during close(): " + ex.toString(), AlertType.ERROR);
                }
            }
        }

    }

    private NDEFTagConnection getNDEFTAGConnection(TargetProperties[] tProp) {
        for (int j = 0; j < tProp.length; j++) {
            Class[] connections = tProp[j].getConnectionNames();
            if (connections != null) {
                for (int i = 0; i < connections.length; i++) {
                    if (connections[i].getName().equals(
                            "javax.microedition.contactless.ndef.NDEFTagConnection")) {
                        try {
                            return (NDEFTagConnection) Connector.open(tProp[0].getUrl(connections[i]));
                        } catch (Exception e) {
                            displayAlert("Error writting NDEF: " + e.toString(), AlertType.ERROR);
                        }
                    }
                }
            }
        }
        return null;
    }

    /**
     * Implementation of the Call-Back function of the CommandListener
     * @param command: command key pressed
     * @param displayable: associated displayable Object
     */
    public void commandAction(Command command, Displayable displayable) {
        if (command == exitCommand) {
            DiscoveryManager dm = DiscoveryManager.getInstance();
            dm.removeTargetListener(this, TargetType.NDEF_TAG);
            destroyApp(false);
            notifyDestroyed();
        }
    }

    private void displayAlert(String error, AlertType type) {
        Alert err = new Alert(form.getTitle(), error, null, type);
        Display.getDisplay(this).setCurrent(err, form);
    }
}
